@@ -1,6 +1,5 @@
 #!/usr/bin/env python 
 #-----------------------------------------------------------------------
-# $Id$
 #
 # Description: 
 #
@@ -57,10 +56,15 @@ from datetime import datetime, timedelta
 from multiprocessing import Lock, Process, Queue , cpu_count 
 from multiprocessing.queues import Empty as MPQEmpty
 from multiprocessing.queues import Full as MPQFull
-
+from django.utils.encoding import smart_text
 try:    import cPickle as pickle 
 except: import pickle 
-
+try:
+    # Python 2
+    xrange
+except NameError:
+    # Python 3, xrange is now named range
+    xrange = range
 #-------------------------------------------------------------------------------
 
 QUEUE_EMPTY_QUERY_DELAY=1.5 # time in seconds of next query to empty queue 
@@ -207,10 +211,10 @@ def taskDispatch( taskID , threadID ) :
     except (KeyboardInterrupt,SystemExit): raise 
     except Exception as e : 
 
-        pStatus.setFailure( unicode(e) ) 
+        pStatus.setFailure( smart_text((e) ) 
 
         # finish the task 
-        error( "[%3.3i] %s " % ( threadID , unicode(e) ) ) 
+        error( "[%3.3i] %s " % ( threadID , smart_text((e) ) ) 
 
 
 #-------------------------------------------------------------------------------
